// File autogenerated

// declaration include
#include "connection.h"

// Qt includes
#include <qdom.h>
#include <qstring.h>

// Qt D-Bus includes
#include <dbus/qdbusdata.h>
#include <dbus/qdbusdatamap.h>
#include <dbus/qdbuserror.h>
#include <dbus/qdbusmessage.h>


namespace DBus
{

void Connection::buildIntrospectionData(QDomElement& interfaceElement)
{
    interfaceElement.setAttribute("name", "org.freedesktop.NetworkManagerSettings.Connection");

    QDomDocument document = interfaceElement.ownerDocument();
    QDomElement methodElement = document.createElement("method");
    methodElement.setAttribute("name", "GetID");

    QDomElement argumentElement = document.createElement("arg");
    argumentElement.setAttribute("name",      "id");
    argumentElement.setAttribute("type",      "s");
    argumentElement.setAttribute("direction", "out");
    methodElement.appendChild(argumentElement);

    interfaceElement.appendChild(methodElement);

    methodElement = document.createElement("method");
    methodElement.setAttribute("name", "Update");

    argumentElement = document.createElement("arg");
    argumentElement.setAttribute("name",      "properties");
    argumentElement.setAttribute("type",      "a{sa{sv}}");
    argumentElement.setAttribute("direction", "in");
    methodElement.appendChild(argumentElement);

    interfaceElement.appendChild(methodElement);

    methodElement = document.createElement("method");
    methodElement.setAttribute("name", "Delete");

    interfaceElement.appendChild(methodElement);

    methodElement = document.createElement("method");
    methodElement.setAttribute("name", "GetSettings");

    argumentElement = document.createElement("arg");
    argumentElement.setAttribute("name",      "settings");
    argumentElement.setAttribute("type",      "a{sa{sv}}");
    argumentElement.setAttribute("direction", "out");
    methodElement.appendChild(argumentElement);

    interfaceElement.appendChild(methodElement);

    methodElement = document.createElement("signal");
    methodElement.setAttribute("name", "Updated");

    argumentElement = document.createElement("arg");
    argumentElement.setAttribute("name",      "settings");
    argumentElement.setAttribute("type",      "a{sa{sv}}");
    argumentElement.setAttribute("direction", "in");
    methodElement.appendChild(argumentElement);

    interfaceElement.appendChild(methodElement);

    methodElement = document.createElement("signal");
    methodElement.setAttribute("name", "Removed");

    interfaceElement.appendChild(methodElement);
}

bool Connection::emitUpdated(const QDBusDataMap<QString>& settings)
{
    QString path = objectPath();
    Q_ASSERT(!path.isEmpty());

    QDBusMessage message = QDBusMessage::signal(path, "org.freedesktop.NetworkManagerSettings.Connection", "Updated");

    message << QDBusData::fromStringKeyMap(settings);

    return handleSignalSend(message);
}

bool Connection::emitRemoved()
{
    QString path = objectPath();
    Q_ASSERT(!path.isEmpty());

    QDBusMessage message = QDBusMessage::signal(path, "org.freedesktop.NetworkManagerSettings.Connection", "Removed");


    return handleSignalSend(message);
}


QDBusMessage Connection::callGetID(const QDBusMessage& message)
{
    QDBusError   error;
    QDBusMessage reply;

    QString _id;

    if (GetID(_id, error))
    {
        reply = QDBusMessage::methodReply(message);
        reply << QDBusData::fromString(_id);
    }
    else
    {
        if (!error.isValid())
        {
            qWarning("Call to implementation of DBus::Connection::GetID returned 'false' but error object is not valid!");

            error = QDBusError::stdFailed("DBus.Connection.GetID execution failed");
        }

        reply = QDBusMessage::methodError(message, error);
    }

    return reply;
}

QDBusMessage Connection::callUpdate(const QDBusMessage& message)
{
    QDBusError   error;
    QDBusMessage reply;

    QDBusDataMap<QString> _properties = message[0].toStringKeyMap();

    if (Update(_properties, error))
    {
        reply = QDBusMessage::methodReply(message);
    }
    else
    {
        if (!error.isValid())
        {
            qWarning("Call to implementation of DBus::Connection::Update returned 'false' but error object is not valid!");

            error = QDBusError::stdFailed("DBus.Connection.Update execution failed");
        }

        reply = QDBusMessage::methodError(message, error);
    }

    return reply;
}

QDBusMessage Connection::callDelete(const QDBusMessage& message)
{
    QDBusError   error;
    QDBusMessage reply;


    if (Delete(error))
    {
        reply = QDBusMessage::methodReply(message);
    }
    else
    {
        if (!error.isValid())
        {
            qWarning("Call to implementation of DBus::Connection::Delete returned 'false' but error object is not valid!");

            error = QDBusError::stdFailed("DBus.Connection.Delete execution failed");
        }

        reply = QDBusMessage::methodError(message, error);
    }

    return reply;
}

QDBusMessage Connection::callGetSettings(const QDBusMessage& message)
{
    QDBusError   error;
    QDBusMessage reply;

    QDBusDataMap<QString> _settings;

    if (GetSettings(_settings, error))
    {
        reply = QDBusMessage::methodReply(message);
        reply << QDBusData::fromStringKeyMap(_settings);
    }
    else
    {
        if (!error.isValid())
        {
            qWarning("Call to implementation of DBus::Connection::GetSettings returned 'false' but error object is not valid!");

            error = QDBusError::stdFailed("DBus.Connection.GetSettings execution failed");
        }

        reply = QDBusMessage::methodError(message, error);
    }

    return reply;
}

bool Connection::handleMethodCall(const QDBusMessage& message)
{
    if (message.interface() != "org.freedesktop.NetworkManagerSettings.Connection") return false;

    if (message.member() == "GetID")
    {
        QDBusMessage reply = callGetID(message);
        handleMethodReply(reply);

        return true;
    }

    if (message.member() == "Update")
    {
        QDBusMessage reply = callUpdate(message);
        handleMethodReply(reply);

        return true;
    }

    if (message.member() == "Delete")
    {
        QDBusMessage reply = callDelete(message);
        handleMethodReply(reply);

        return true;
    }

    if (message.member() == "GetSettings")
    {
        QDBusMessage reply = callGetSettings(message);
        handleMethodReply(reply);

        return true;
    }

    return false; 
}

}; // namespace DBus

// Qt includes
#include <qdom.h>
#include <qstring.h>
#include <qstringlist.h>

// Qt D-Bus includes
#include <dbus/qdbusdata.h>
#include <dbus/qdbusdatalist.h>
#include <dbus/qdbusdatamap.h>
#include <dbus/qdbuserror.h>
#include <dbus/qdbusmessage.h>


namespace DBus
{

void Secrets::buildIntrospectionData(QDomElement& interfaceElement)
{
    interfaceElement.setAttribute("name", "org.freedesktop.NetworkManagerSettings.Connection.Secrets");

    QDomDocument document = interfaceElement.ownerDocument();
    QDomElement methodElement = document.createElement("method");
    methodElement.setAttribute("name", "GetSecrets");

    QDomElement argumentElement = document.createElement("arg");
    argumentElement.setAttribute("name",      "setting_name");
    argumentElement.setAttribute("type",      "s");
    argumentElement.setAttribute("direction", "in");
    methodElement.appendChild(argumentElement);

    argumentElement = document.createElement("arg");
    argumentElement.setAttribute("name",      "hints");
    argumentElement.setAttribute("type",      "as");
    argumentElement.setAttribute("direction", "in");
    methodElement.appendChild(argumentElement);

    argumentElement = document.createElement("arg");
    argumentElement.setAttribute("name",      "request_new");
    argumentElement.setAttribute("type",      "b");
    argumentElement.setAttribute("direction", "in");
    methodElement.appendChild(argumentElement);

    argumentElement = document.createElement("arg");
    argumentElement.setAttribute("name",      "secrets");
    argumentElement.setAttribute("type",      "a{sa{sv}}");
    argumentElement.setAttribute("direction", "out");
    methodElement.appendChild(argumentElement);

    interfaceElement.appendChild(methodElement);
}

void Secrets::GetSecretsAsyncReply(int asyncCallId, const QDBusDataMap<QString>& secrets)

{
    QMap<int, QDBusMessage>::iterator findIt = m_asyncCalls.find(asyncCallId);
    if (findIt == m_asyncCalls.end()) return;

    QDBusMessage call = findIt.data();
    m_asyncCalls.erase(findIt);

    QDBusMessage reply = QDBusMessage::methodReply(call);
    reply << QDBusData::fromStringKeyMap(secrets);

    handleMethodReply(reply);
}

void Secrets::GetSecretsAsyncError(int asyncCallId, const QDBusError& error)
{
    QMap<int, QDBusMessage>::iterator findIt = m_asyncCalls.find(asyncCallId);
    if (findIt == m_asyncCalls.end()) return;

    QDBusMessage call = findIt.data();
    m_asyncCalls.erase(findIt);

    QDBusMessage reply = QDBusMessage::methodError(call, error);
    handleMethodReply(reply);
}

void Secrets::callGetSecretsAsync(const QDBusMessage& message)
{
    QString _setting_name = message[0].toString();
    QStringList _hints = message[1].toList().toQStringList();
    bool _request_new = message[2].toBool();

    int _asyncCallId = 0;
    while (m_asyncCalls.find(_asyncCallId) != m_asyncCalls.end())
    {
        ++_asyncCallId;
    }
    m_asyncCalls.insert(_asyncCallId, message);

    GetSecretsAsync(_asyncCallId, _setting_name, _hints, _request_new);

    return;
}

bool Secrets::handleMethodCall(const QDBusMessage& message)
{
    if (message.interface() != "org.freedesktop.NetworkManagerSettings.Connection.Secrets") return false;

    if (message.member() == "GetSecrets")
    {
        callGetSecretsAsync(message);

        return true;
    }

    return false; 
}

}; // namespace DBus

// End of File

