// File autogenerated

// declaration include
#include "networkmanagerproxy.h"

// Qt includes
#include <qmap.h>
#include <qstring.h>
#include <qvaluelist.h>

// Qt D-Bus includes
#include <dbus/qdbusconnection.h>
#include <dbus/qdbusdata.h>
#include <dbus/qdbusdatalist.h>
#include <dbus/qdbusdatamap.h>
#include <dbus/qdbuserror.h>
#include <dbus/qdbusmessage.h>
#include <dbus/qdbusobjectpath.h>
#include <dbus/qdbusproxy.h>
#include <dbus/qdbusvariant.h>


namespace DBus
{

NetworkManagerProxy::NetworkManagerProxy(const QString& service, const QString& path, QObject* parent, const char* name)
    : QObject(parent, name),
      m_baseProxy(new QDBusProxy())
{
    m_baseProxy->setInterface("org.freedesktop.NetworkManager");
    m_baseProxy->setPath(path);
    m_baseProxy->setService(service);

    QObject::connect(m_baseProxy, SIGNAL(dbusSignal(const QDBusMessage&)),
                     this,        SLOT(slotHandleDBusSignal(const QDBusMessage&)));
    QObject::connect(m_baseProxy, SIGNAL(asyncReply(int, const QDBusMessage&)),
                     this,        SLOT(slotHandleAsyncReply(int, const QDBusMessage&)));
}

NetworkManagerProxy::~NetworkManagerProxy()
{
    delete m_baseProxy;
}

void NetworkManagerProxy::setConnection(const QDBusConnection& connection)
{
    m_baseProxy->setConnection(connection);
}

bool NetworkManagerProxy::GetDevices(QValueList<QDBusObjectPath>& devices, QDBusError& error)
{
    QValueList<QDBusData> parameters;


    QDBusMessage reply = m_baseProxy->sendWithReply("GetDevices", parameters, &error);

    if (reply.type() != QDBusMessage::ReplyMessage) return false;

    if (reply.count() != 1) return false;

    bool ok = false;

    bool subOK = false;

    devices = reply.front().toList(&ok).toObjectPathList(&subOK);
    if (!subOK) return false;
    if (!ok) return false;

    return true;
}

bool NetworkManagerProxy::DeactivateConnection(const QDBusObjectPath& active_connection, QDBusError& error)
{
    QValueList<QDBusData> parameters;

    parameters << QDBusData::fromObjectPath(active_connection);

    QDBusMessage reply = m_baseProxy->sendWithReply("DeactivateConnection", parameters, &error);

    if (reply.type() != QDBusMessage::ReplyMessage) return false;
    return true;
}

bool NetworkManagerProxy::Sleep(bool sleep, QDBusError& error)
{
    QValueList<QDBusData> parameters;

    parameters << QDBusData::fromBool(sleep);

    QDBusMessage reply = m_baseProxy->sendWithReply("Sleep", parameters, &error);

    if (reply.type() != QDBusMessage::ReplyMessage) return false;
    return true;
}

bool NetworkManagerProxy::sleep(QDBusError& error)
{
    QValueList<QDBusData> parameters;


    QDBusMessage reply = m_baseProxy->sendWithReply("sleep", parameters, &error);

    if (reply.type() != QDBusMessage::ReplyMessage) return false;
    return true;
}

bool NetworkManagerProxy::wake(QDBusError& error)
{
    QValueList<QDBusData> parameters;


    QDBusMessage reply = m_baseProxy->sendWithReply("wake", parameters, &error);

    if (reply.type() != QDBusMessage::ReplyMessage) return false;
    return true;
}

bool NetworkManagerProxy::state(Q_UINT32& state, QDBusError& error)
{
    QValueList<QDBusData> parameters;


    QDBusMessage reply = m_baseProxy->sendWithReply("state", parameters, &error);

    if (reply.type() != QDBusMessage::ReplyMessage) return false;

    if (reply.count() != 1) return false;

    bool ok = false;

    state = reply.front().toUInt32(&ok);
    if (!ok) return false;

    return true;
}

bool NetworkManagerProxy::ActivateConnectionAsync(int& asyncCallId, const QString& service_name, const QDBusObjectPath& connection, const QDBusObjectPath& device, const QDBusObjectPath& specific_object, QDBusError& error)
{
    QValueList<QDBusData> parameters;

    parameters << QDBusData::fromString(service_name);
    parameters << QDBusData::fromObjectPath(connection);
    parameters << QDBusData::fromObjectPath(device);
    parameters << QDBusData::fromObjectPath(specific_object);

    asyncCallId = m_baseProxy->sendWithAsyncReply("ActivateConnection", parameters);

    if (asyncCallId != 0) m_asyncCalls[asyncCallId] = "ActivateConnection";

    return (asyncCallId != 0);
}

void NetworkManagerProxy::setDBusProperty(const QString& name, const QDBusVariant& value, QDBusError& error)
{
    QDBusConnection connection = m_baseProxy->connection();

    QDBusMessage message = QDBusMessage::methodCall(m_baseProxy->service(), m_baseProxy->path(), "org.freedesktop.DBus.Properties", "Set");

    message << QDBusData::fromString(m_baseProxy->interface());
    message << QDBusData::fromString(name);
    message << QDBusData::fromVariant(value);

    connection.sendWithReply(message, &error);
}

QDBusVariant NetworkManagerProxy::getDBusProperty(const QString& name, QDBusError& error) const
{
    QDBusConnection connection = m_baseProxy->connection();

    QDBusMessage message = QDBusMessage::methodCall(m_baseProxy->service(), m_baseProxy->path(), "org.freedesktop.DBus.Properties", "Get");

    message << QDBusData::fromString(m_baseProxy->interface());
    message << QDBusData::fromString(name);

    QDBusMessage reply = connection.sendWithReply(message, &error);

    if (reply.type() != QDBusMessage::ReplyMessage) return QDBusVariant();
    if (reply.count() != 1) return QDBusVariant();

    bool ok = false;
    QDBusVariant value = reply.front().toVariant(&ok);
    if (!ok) return QDBusVariant();

    return value;
}

void NetworkManagerProxy::setWirelessEnabled(bool value, QDBusError& error)
{
    QDBusVariant variant;
    variant.value = QDBusData::fromBool(value);
    variant.signature = "b";

    setDBusProperty("WirelessEnabled", variant, error);
}

bool NetworkManagerProxy::getWirelessEnabled(QDBusError& error) const
{
    QDBusVariant variant = getDBusProperty("WirelessEnabled", error);

    if (error.isValid()) return bool();

    bool ok = false;

    bool result =  variant.value.toBool(&ok);
    if (!ok) {}

    return result;
}

bool NetworkManagerProxy::getWirelessHardwareEnabled(QDBusError& error) const
{
    QDBusVariant variant = getDBusProperty("WirelessHardwareEnabled", error);

    if (error.isValid()) return bool();

    bool ok = false;

    bool result =  variant.value.toBool(&ok);
    if (!ok) {}

    return result;
}

QValueList<QDBusObjectPath> NetworkManagerProxy::getActiveConnections(QDBusError& error) const
{
    QDBusVariant variant = getDBusProperty("ActiveConnections", error);

    if (error.isValid()) return QValueList<QDBusObjectPath>();

    bool ok = false;

    bool subOK = false;

    QValueList<QDBusObjectPath> result =  variant.value.toList(&ok).toObjectPathList(&subOK);
    if (!subOK) {}
    if (!ok) {}

    return result;
}

Q_UINT32 NetworkManagerProxy::getState(QDBusError& error) const
{
    QDBusVariant variant = getDBusProperty("State", error);

    if (error.isValid()) return Q_UINT32();

    bool ok = false;

    Q_UINT32 result =  variant.value.toUInt32(&ok);
    if (!ok) {}

    return result;
}

void NetworkManagerProxy::slotHandleDBusSignal(const QDBusMessage& message)
{
    if (message.member() == "StateChanged")
    {
        Q_UINT32 _state = message[0].toUInt32();

        emit StateChanged(_state);
    }
    else if (message.member() == "PropertiesChanged")
    {
        QMap<QString, QDBusVariant> _properties = message[0].toStringKeyMap().toVariantMap();

        emit PropertiesChanged(_properties);
    }
    else if (message.member() == "DeviceAdded")
    {
        QDBusObjectPath _device_path = message[0].toObjectPath();

        emit DeviceAdded(_device_path);
    }
    else if (message.member() == "DeviceRemoved")
    {
        QDBusObjectPath _device_path = message[0].toObjectPath();

        emit DeviceRemoved(_device_path);
    }
    else if (message.member() == "StateChange")
    {
        Q_UINT32 _state = message[0].toUInt32();

        emit StateChange(_state);
    }
}

void NetworkManagerProxy::slotHandleAsyncReply(int asyncCallId, const QDBusMessage& message)
{
    QMap<int, QString>::iterator findIt = m_asyncCalls.find(asyncCallId);
    if (findIt == m_asyncCalls.end()) return;

    const QString signalName = findIt.data();
    m_asyncCalls.erase(findIt);

    if (signalName == "ActivateConnection")
    {
        int _asyncCallId = asyncCallId;
        QDBusObjectPath _active_connection = message[0].toObjectPath();

        emit ActivateConnectionAsyncReply(_asyncCallId, _active_connection);
    }
}

}; // namespace DBus

// End of File

