// File autogenerated

// declaration include
#include "wirelessproxy.h"

// Qt includes
#include <qmap.h>
#include <qstring.h>
#include <qvaluelist.h>

// Qt D-Bus includes
#include <dbus/qdbusconnection.h>
#include <dbus/qdbusdata.h>
#include <dbus/qdbusdatalist.h>
#include <dbus/qdbusdatamap.h>
#include <dbus/qdbuserror.h>
#include <dbus/qdbusmessage.h>
#include <dbus/qdbusobjectpath.h>
#include <dbus/qdbusproxy.h>
#include <dbus/qdbusvariant.h>


namespace DBus
{

WirelessDeviceProxy::WirelessDeviceProxy(const QString& service, const QString& path, QObject* parent, const char* name)
    : QObject(parent, name),
      m_baseProxy(new QDBusProxy())
{
    m_baseProxy->setInterface("org.freedesktop.NetworkManager.Device.Wireless");
    m_baseProxy->setPath(path);
    m_baseProxy->setService(service);

    QObject::connect(m_baseProxy, SIGNAL(dbusSignal(const QDBusMessage&)),
                     this,        SLOT(slotHandleDBusSignal(const QDBusMessage&)));
}

WirelessDeviceProxy::~WirelessDeviceProxy()
{
    delete m_baseProxy;
}

void WirelessDeviceProxy::setConnection(const QDBusConnection& connection)
{
    m_baseProxy->setConnection(connection);
}

bool WirelessDeviceProxy::GetAccessPoints(QValueList<QDBusObjectPath>& access_points, QDBusError& error)
{
    QValueList<QDBusData> parameters;


    QDBusMessage reply = m_baseProxy->sendWithReply("GetAccessPoints", parameters, &error);

    if (reply.type() != QDBusMessage::ReplyMessage) return false;

    if (reply.count() != 1) return false;

    bool ok = false;

    bool subOK = false;

    access_points = reply.front().toList(&ok).toObjectPathList(&subOK);
    if (!subOK) return false;
    if (!ok) return false;

    return true;
}

void WirelessDeviceProxy::setDBusProperty(const QString& name, const QDBusVariant& value, QDBusError& error)
{
    QDBusConnection connection = m_baseProxy->connection();

    QDBusMessage message = QDBusMessage::methodCall(m_baseProxy->service(), m_baseProxy->path(), "org.freedesktop.DBus.Properties", "Set");

    message << QDBusData::fromString(m_baseProxy->interface());
    message << QDBusData::fromString(name);
    message << QDBusData::fromVariant(value);

    connection.sendWithReply(message, &error);
}

QDBusVariant WirelessDeviceProxy::getDBusProperty(const QString& name, QDBusError& error) const
{
    QDBusConnection connection = m_baseProxy->connection();

    QDBusMessage message = QDBusMessage::methodCall(m_baseProxy->service(), m_baseProxy->path(), "org.freedesktop.DBus.Properties", "Get");

    message << QDBusData::fromString(m_baseProxy->interface());
    message << QDBusData::fromString(name);

    QDBusMessage reply = connection.sendWithReply(message, &error);

    if (reply.type() != QDBusMessage::ReplyMessage) return QDBusVariant();
    if (reply.count() != 1) return QDBusVariant();

    bool ok = false;
    QDBusVariant value = reply.front().toVariant(&ok);
    if (!ok) return QDBusVariant();

    return value;
}

QString WirelessDeviceProxy::getHwAddress(QDBusError& error) const
{
    QDBusVariant variant = getDBusProperty("HwAddress", error);

    if (error.isValid()) return QString();

    bool ok = false;

    QString result =  variant.value.toString(&ok);
    if (!ok) {}

    return result;
}

Q_UINT32 WirelessDeviceProxy::getMode(QDBusError& error) const
{
    QDBusVariant variant = getDBusProperty("Mode", error);

    if (error.isValid()) return Q_UINT32();

    bool ok = false;

    Q_UINT32 result =  variant.value.toUInt32(&ok);
    if (!ok) {}

    return result;
}

Q_UINT32 WirelessDeviceProxy::getBitrate(QDBusError& error) const
{
    QDBusVariant variant = getDBusProperty("Bitrate", error);

    if (error.isValid()) return Q_UINT32();

    bool ok = false;

    Q_UINT32 result =  variant.value.toUInt32(&ok);
    if (!ok) {}

    return result;
}

QDBusObjectPath WirelessDeviceProxy::getActiveAccessPoint(QDBusError& error) const
{
    QDBusVariant variant = getDBusProperty("ActiveAccessPoint", error);

    if (error.isValid()) return QDBusObjectPath();

    bool ok = false;

    QDBusObjectPath result =  variant.value.toObjectPath(&ok);
    if (!ok) {}

    return result;
}

Q_UINT32 WirelessDeviceProxy::getWirelessCapabilities(QDBusError& error) const
{
    QDBusVariant variant = getDBusProperty("WirelessCapabilities", error);

    if (error.isValid()) return Q_UINT32();

    bool ok = false;

    Q_UINT32 result =  variant.value.toUInt32(&ok);
    if (!ok) {}

    return result;
}

void WirelessDeviceProxy::slotHandleDBusSignal(const QDBusMessage& message)
{
    if (message.member() == "PropertiesChanged")
    {
        QMap<QString, QDBusVariant> _properties = message[0].toStringKeyMap().toVariantMap();

        emit PropertiesChanged(_properties);
    }
    else if (message.member() == "AccessPointAdded")
    {
        QDBusObjectPath _access_point = message[0].toObjectPath();

        emit AccessPointAdded(_access_point);
    }
    else if (message.member() == "AccessPointRemoved")
    {
        QDBusObjectPath _access_point = message[0].toObjectPath();

        emit AccessPointRemoved(_access_point);
    }
}

}; // namespace DBus

// End of File

